/*
 * Copyright 2019 ConsenSys AG.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 */
package tech.pegasys.pantheon.config;

import java.util.Map;
import java.util.OptionalLong;

import com.google.common.collect.ImmutableMap;
import io.vertx.core.json.JsonObject;

public class EthashConfigOptions {

  public static final EthashConfigOptions DEFAULT = new EthashConfigOptions(new JsonObject());;
  private final JsonObject ethashConfigRoot;

  EthashConfigOptions(final JsonObject ethashConfigRoot) {
    this.ethashConfigRoot = ethashConfigRoot;
  }

  public OptionalLong getFixedDifficulty() {
    return ConfigUtil.getOptionalLong(ethashConfigRoot, "fixeddifficulty");
  }

  Map<String, Object> asMap() {
    final ImmutableMap.Builder<String, Object> builder = ImmutableMap.builder();
    getFixedDifficulty().ifPresent(l -> builder.put("fixeddifficulty", l));
    return builder.build();
  }
}
