/*
 * Copyright 2018 ConsenSys AG.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 */
package tech.pegasys.pantheon.ethereum.p2p.wire.messages;

import tech.pegasys.pantheon.ethereum.p2p.api.MessageData;
import tech.pegasys.pantheon.ethereum.p2p.wire.AbstractMessageData;
import tech.pegasys.pantheon.ethereum.p2p.wire.PeerInfo;
import tech.pegasys.pantheon.ethereum.rlp.BytesValueRLPOutput;
import tech.pegasys.pantheon.ethereum.rlp.RLP;
import tech.pegasys.pantheon.util.bytes.BytesValue;

public final class HelloMessage extends AbstractMessageData {

  private HelloMessage(final BytesValue data) {
    super(data);
  }

  public static HelloMessage create(final PeerInfo peerInfo) {
    final BytesValueRLPOutput out = new BytesValueRLPOutput();
    peerInfo.writeTo(out);

    return new HelloMessage(out.encoded());
  }

  public static HelloMessage readFrom(final MessageData message) {
    if (message instanceof HelloMessage) {
      return (HelloMessage) message;
    }
    final int code = message.getCode();
    if (code != WireMessageCodes.HELLO) {
      throw new IllegalArgumentException(
          String.format("Message has code %d and thus is not a HelloMessage.", code));
    }
    return new HelloMessage(message.getData());
  }

  @Override
  public int getCode() {
    return WireMessageCodes.HELLO;
  }

  public PeerInfo getPeerInfo() {
    return PeerInfo.readFrom(RLP.input(data));
  }

  @Override
  public String toString() {
    return "HelloMessage{" + "data=" + data + '}';
  }
}
